
// per_tests.cpp
// This set of routines test the Packed Encoding Rule functionality.
//  There will be numerous references to the PER requirements document.
//

int Perform_Per_Tests()
{


}		// END Perform_Per_Tests()


// EOF per_tests.cpp