
#ifndef _snaccCder_h_
#define _snaccCder_h_

#define DEncTag1 BEncTag1
#define DEncAsnOidContent BEncAsnOidContent
#define DDecAsnOidContent BDecAsnOidContent
#define DDecTag BDecTag
#define DEncAsnOctsContent BEncAsnOctsContent
#define DEncAsnBoolContent BEncAsnBoolContent
#define DDecAsnBoolContent BDecAsnBoolContent
#define DDecAsnOctsContent BDecAsnOctsContent
#define DEncAsnIntContent BEncAsnIntContent
#define DDecAsnIntContent BDecAsnIntContent
#define DEncAsnEnumContent BEncAsnEnumContent
#define DDecAsnEnumContent BDecAsnEnumContent
#define DEncAsnRealContent BEncAsnRealContent
#define DEncAsnNullContent BEncAsnNullContent
#define DDecAsnNullContent BDecAsnNullContent
#define DDecAsnRealContent BDecAsnRealContent

#define DEncAsnAnyDefinedBy BEncAsnAny
#define DDecAsnAnyDefinedBy BDecAsnAny
#define DEncBMPStringContent DEncAsnOctsContent
#define DEncUTF8StringContent DEncAsnOctsContent
#define DEncUniversalStringContent DEncAsnOctsContent
#define DEncPrintableStringContent DEncAsnOctsContent
#define DEncTeletexStringContent DEncAsnOctsContent
#define DDecBMPStringContent DDecAsnOctsContent
#define DDecUTF8StringContent DDecAsnOctsContent
#define DDecUniversalStringContent DDecAsnOctsContent
#define DDecPrintableStringContent DDecAsnOctsContent
#define DDecTeletexStringContent DDecAsnOctsContent




#endif /* conditional include */

